/*
 * timeing.h
 *
 *  Created on: 08.09.2010
 *      Author: kobitzsch
 */

#ifndef TIMER_H_
#define TIMER_H_

#ifndef WIN32
#include <time.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/resource.h>
#include <stdint.h>
#endif

class ClockCounterTimer{
public:
	ClockCounterTimer( double frequency_ )
	: frequency( frequency_ )
	{}

	inline void start(){
		count_at_start = rdtsc();
	}

	inline double stop(){
		count_at_stop = rdtsc();
		return (double)(count_at_stop - count_at_start) / frequency;
	}

	inline double getTime() const {
		return (double)(count_at_stop - count_at_start) / frequency;
	}
protected:
	double frequency;
	unsigned long long count_at_start;
	unsigned long long count_at_stop;

	inline unsigned long long rdtsc() {
	uint32_t lo, hi;
	__asm__ __volatile__ (      // serialize
	"xorl %%eax,%%eax \n        cpuid"
	::: "%rax", "%rbx", "%rcx", "%rdx");
	/* We cannot use "=A", since this would use %rax on x86_64 and return only the lower 32bits of the TSC */
	__asm__ __volatile__ ("rdtsc" : "=a" (lo), "=d" (hi));
	return (uint64_t)hi << 32 | lo;
	}
};

#endif /* TIMER_H_ */
